#include "CUDAGridPartition.h"

#include <cuda_runtime.h> 
#include <cutil.h>
#include <cutil_math.h>


/// Device Code
texture<float2,3,cudaReadModeElementType> MinMaxTex;

__constant__ int c_bsize[3];
__constant__ int c_bnum[3];

static __device__
float2 d_GetMinMax(int x,int y,int z)
{
	return tex3D(MinMaxTex,(float)x+0.5f,(float)y+0.5f,(float)z+0.5f);
}

static __device__
int ConvertToBlockIndex(int voxelIndex,int dimId)
{
	return voxelIndex/c_bsize[dimId];
}

static __device__
int ConvertToVoxelIndex(int blockIndex,int dimId)
{
	return blockIndex*c_bsize[dimId];
}


/// Host Code
static bool GridPartitionPrepare(CUDAGridPartition *gp)
{
	if (!gp) return false;

	MinMaxTex.normalized = false;  
    MinMaxTex.filterMode = cudaFilterModePoint;
    MinMaxTex.addressMode[0] = cudaAddressModeClamp; 
    MinMaxTex.addressMode[1] = cudaAddressModeClamp;
	MinMaxTex.addressMode[2] = cudaAddressModeClamp;

	cudaChannelFormatDesc channelDesc;

	cudaArray* d_MinMaxArray=gp->m_d_MinMaxArray;
	
	CUDA_SAFE_CALL(cudaGetChannelDesc(&channelDesc, d_MinMaxArray));
	CUDA_SAFE_CALL(cudaBindTextureToArray(MinMaxTex, d_MinMaxArray, channelDesc));

	int bsize[3];
	int bnum[3];

	bsize[0]=gp->m_bsizeX;
	bsize[1]=gp->m_bsizeY;
	bsize[2]=gp->m_bsizeZ;

	bnum[0]=gp->m_bnumX;
	bnum[1]=gp->m_bnumY;
	bnum[2]=gp->m_bnumZ;

	CUDA_SAFE_CALL(cudaMemcpyToSymbol(c_bsize, bsize, sizeof(int)*3,0,cudaMemcpyHostToDevice));
	CUDA_SAFE_CALL(cudaMemcpyToSymbol(c_bnum, bnum, sizeof(int)*3,0,cudaMemcpyHostToDevice));

	return true;
}

static void GridPartitionClear()
{
	CUDA_SAFE_CALL(cudaUnbindTexture(MinMaxTex));
}
