#include "MIPRayCasterParam.h"

static __device__ float d_GetScalarValue(const float4& pos);

#include <cuda_runtime.h> 
#include <cutil.h>
#include <cutil_math.h>


__constant__ float c_mip_scale;
__constant__ float c_mip_shift;

static __device__
float d_func(float x,float a,float b,float c,float d)
{
	return x*(x*(x*a+b)+c)+d;
}

static __device__
float d_GetMax(float a,float b,float c,float d)
{
	float maxI=d;
	maxI=max(maxI,a+b+c+d);

	float y;

	if (a==0.0f)
	{
		if (b!=0.0f)
		{
			float x=-c/(2.0f*b);
			if (x>0.0f && x<1.0f)
			{
				y=d_func(x,a,b,c,d);
				maxI=max(maxI,y);
			}
		}
	}
	else
	{
		float _b=b/(3.0f*a);
		float _c=c/(3.0f*a);
		
		float delta=_b*_b-_c;		
		if (delta>0)
		{
			delta=sqrtf(delta);
			float r1,r2;
			if (_b>0.0f) 
			{
				r1=-_b-delta;
				r2=_c/r1;
			}
			else 
			{
				r2=-_b+delta;
				r1=_c/r2;
			}
			if (r1>0.0f && r1<1.0f)
			{
				y=d_func(r1,a,b,c,d);
				maxI=max(maxI,y);
			}
			if (r2>0.0f && r2<1.0f)
			{
				y=d_func(r2,a,b,c,d);			
				maxI=max(maxI,y);
			}
		}
	}
	
	return maxI;

}

static bool MIPRayCasterPrepare(MIPRayCasterParam *RayCaster)
{
	if (!RayCaster) return false;

	float windowWidth=RayCaster->m_windowWidth;
	float windowCenter=RayCaster->m_windowCenter;

	float mip_scale=1.0f/windowWidth;
	float mip_shift=0.5-windowCenter/windowWidth;
	CUDA_SAFE_CALL(cudaMemcpyToSymbol(c_mip_scale, &mip_scale, sizeof(float),0,cudaMemcpyHostToDevice));
	CUDA_SAFE_CALL(cudaMemcpyToSymbol(c_mip_shift, &mip_shift, sizeof(float),0,cudaMemcpyHostToDevice));

	
	return true;
}
