#ifndef __EndoscopySliceScene_h
#define __EndoscopySliceScene_h

#include "AbstractScene.h"
#include "EndoscopySliceRendering.h"

class SceneParam;

class VOLTK_SCENE_API EndoscopySliceScene : public AbstractScene
{
public:
	EndoscopySliceScene();
	virtual ~EndoscopySliceScene();

	virtual void Render();
	virtual void Resize(int width, int height);

	void SetVolume(CoherentVolume* vol);
	CoherentVolume* GetVolume();

	void SetVolumeInfo(int dims[3], float spacings[3]);

	void SetHorizontial() { m_Horizontial=true; } 
	void SetVertical() { m_Horizontial=false; } 
	bool IsHorizontial() { return m_Horizontial;}

	void SetGridSpacing(float spacing) { m_GridSpacing=spacing; }
	float GetGridSpacing() { return m_GridSpacing; }

	void SetSceneParam(SceneParam* sceneParam) { m_SceneParam=sceneParam; }

	SceneParam* GetSceneParam() { return m_SceneParam;}

	void SetWidthCenter(float windowWidth,float windowCenter);
	void SetWindowWidth(float windowWidth);
	void SetWindowCenter(float windowCenter);

	float GetWindowWidth();
	float GetWindowCenter();

	virtual void LButtonDown(int x, int y, retStatus &status);
	virtual void LButtonUp(int x, int y, retStatus &status);

	virtual void MButtonDown(int x, int y, retStatus &status);
	virtual void MButtonUp(int x, int y, retStatus &status);

	virtual void RButtonDown(int x, int y, retStatus &status);
	virtual void RButtonUp(int x, int y, retStatus &status);

	virtual void MouseMove(int x, int y, retStatus &status);
	virtual void Wheel(int value, retStatus &status);

	void SetRotateRadPerPixel(float radPerPixel) { m_RadPerPixel=radPerPixel;}

	float GetRotateRadPerPixel() const { return m_RadPerPixel; }

	EndoscopySliceRendering& GetKernelModule() { return m_module; }


protected:
	int m_OldMouseX,m_OldMouseY;

private:
	EndoscopySliceRendering m_module;
	bool m_Horizontial;
	float m_GridSpacing;

	int m_ViewPort[2];
	SceneParam* m_SceneParam;

	float m_RadPerPixel;

	float m_oldWidth,m_oldCenter;
	bool m_AdjustingWidthCenter;
	bool m_Panning;
	bool m_Turning;

};

#endif