#ifndef __FullVRScene_h
#define __FullVRScene_h

#include "Scene.h"
#include "FullVR.h"
#include "VolumeData.h"
#include "RGBATransferFunction1D.h"

class CoherentVolume;

class VOLTK_SCENE_API FullVRScene : public Scene
{
public:
	FullVRScene();
	virtual ~FullVRScene();
	
	void SetData(VolumeData *data);
	void SetDataModified();

	void SetBackGroundColor(float r,float g, float b);
	void GetBackGroundColor(float& r,float& g,float& b);
	
	RGBATransferFunction1D* GetTransferFunction();
	void SetTransferFunction(RGBATransferFunction1D* tf);

	void SetStdSampleDistance(float stdSampleDistance);
	float GetStdSampleDistance();

	void SetShade(bool shade);
	bool GetShade();

	void ShadeOn();
	void ShadeOff();

	void SetLightDirection(float x,float y,float z);
	void GetLightDirection(float &x,float &y,float &z);

	void SetLightIntensity(float intensity);
	void SetLightColor(float r,float g,float b);

	void SetAmbient(float value);
	float GetAmbient();

	void SetDiffuse(float value);
	float GetDiffuse();

	void SetSpecular(float value);
	float GetSpecular();

	void SetSpecularPower(float value);
	float GetSpecularPower();

	void RotateLightDirection(bool turnOn);

	virtual void LButtonDown(int x, int y, retStatus &status);
	virtual void LButtonUp(int x, int y, retStatus &status);
	virtual void MouseMove(int x, int y, retStatus &status);

	void SetIllustrativeShading(bool illustrative);
	bool GetIllustrativeShading();

	void SetCroppingBounds(const float CroppingBounds[6]);
	void GetCroppingBounds(float CroppingBounds[6]);

	FullVR& GetKernelModule() { return m_module; }

	
protected:
	virtual void RenderContent();
	virtual void GetSize3D(float size[3]);

private:
	VolumeData* m_Data;
	RGBATransferFunction1D* m_TF;

	FullVR m_module;

	bool m_RotateLightDirection;
	bool m_RotatingLightDirection;

	Vector4 m_OldLightDirection;

};

#endif
