#ifndef __IsosurfaceVRScene_h
#define __IsosurfaceVRScene_h

#include "Scene.h"
#include "IsosurfaceVR.h"
#include "VolumeData.h"

class CoherentVolume;

class VOLTK_SCENE_API IsosurfaceVRScene : public Scene
{
public:
	IsosurfaceVRScene();
	virtual ~IsosurfaceVRScene();
	
	void SetData(VolumeData *data);
	void SetDataModified();

	void SetBackGroundColor(float r,float g, float b);	
	void GetBackGroundColor(float& r,float& g,float& b);
	
	void SetIsovalue(float isovalue);
	float GetIsovalue();

	void SetSurfaceColor(float r,float g,float b);
	void GetSurfaceColor(float &r,float &g,float &b);

	void SetLightDirection(float x,float y,float z);
	void SetLightIntensity(float intensity);
	void SetLightColor(float r,float g,float b);

	void SetAmbient(float value);
	float GetAmbient();

	void SetDiffuse(float value);
	float GetDiffuse();

	void SetSpecular(float value);
	float GetSpecular();

	void SetSpecularPower(float value);
	float GetSpecularPower();

	void RotateLightDirection(bool turnOn);

	virtual void LButtonDown(int x, int y, retStatus &status);
	virtual void LButtonUp(int x, int y, retStatus &status);
	virtual void MouseMove(int x, int y, retStatus &status);

	void SetIllustrativeShading(bool illustrative);
	bool GetIllustrativeShading();

	void SetCroppingBounds(const float CroppingBounds[6]);
	void GetCroppingBounds(float CroppingBounds[6]);

	IsosurfaceVR& GetKernelModule() { return m_module; }

protected:
	virtual void RenderContent();
	virtual void GetSize3D(float size[3]);
	
private:
	VolumeData* m_Data;
	IsosurfaceVR m_module;

	bool m_RotateLightDirection;
	bool m_RotatingLightDirection;

	Vector4 m_OldLightDirection;
};

#endif
