#ifndef _VolumeData_h
#define _VolumeData_h

#include <stdio.h>
#include "VolTKStandardIncludes.h"

#ifndef _DataType_
#define _DataType_
struct DataType
{
	bool isFloat;
	bool isSigned;
	int bitsPerSample;
};
#endif

class VOLTK_STANDARD_API VolumeData
{
public:
	static const DataType dtFloat32;
	static const DataType dtInt8;
	static const DataType dtUint8;
	static const DataType dtInt16;
	static const DataType dtUint16;
	static const DataType dtInt32;
	static const DataType dtUint32;

	VolumeData();
	virtual ~VolumeData();

	void Clean();
	void Allocate();

	DataType GetDataType() const;
	void SetDataType(DataType type); 

	void GetDimensions(int *dims) const;
	void SetDimensions(const int *dims);

	void GetSpacings(float *spacings) const;
	void SetSpacings(const float *spacings);

	void* GetData() const;

	void GetMinMaxValue(double &minVal, double &maxVal);

	void WriteRawFile(FILE *fp) const;
	void ReadRawFile(FILE *fp);

	void WriteStructuredFile(const char* filename) const;
	void ReadStructuredFile(const char* filename);


private:
	DataType m_DataType;

	int m_Dims[3];
	float m_Spacings[3];

	void *m_Data;

};

#endif