#include "MIPRayCasterParam.h"
#include "Vector4.h"
#include "Vector3.h"


static inline float GetScalarValue(int x,int y,int z);
static inline float GetScalarValue(Vector4 pos);

static float mip_scale;
static float mip_shift;


#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

static inline float func(float x,float a,float b,float c,float d)
{
	return x*(x*(x*a+b)+c)+d;
}

static inline float GetMax(float a,float b,float c,float d,float x1,float x2)
{
	float maxI=func(x1,a,b,c,d);
	maxI=max(maxI,func(x2,a,b,c,d));

	float y;
	
	if (a==0.0f)
	{
		if (b!=0.0f)
		{
			float x=-c/(2.0f*b);
			if (x>x1 && x<x2) 
			{
				float y=func(x,a,b,c,d);
				maxI=max(maxI,y);
			}
		}
	}
	else
	{
		float _b=b/(3.0f*a);
		float _c=c/(3.0f*a);
		
		float delta=_b*_b-_c;		
		if (delta>0)
		{
			delta=sqrtf(delta);
			float r1,r2;
			if (_b>0.0f) 
			{
				r1=-_b-delta;
				r2=_c/r1;
			}
			else 
			{
				r2=-_b+delta;
				r1=_c/r2;
			}

			if (r1>x1 && r1<x2) 
			{
				y=func(r1,a,b,c,d);
				maxI=max(maxI,y);
			}
			if (r2>x1 && r2<x2) 
			{
				y=func(r2,a,b,c,d);			
				maxI=max(maxI,y);
			}
		}
	}
	
	return maxI;
}

static bool MIPRayCasterPrepare(MIPRayCasterParam *RayCaster)
{
	if (!RayCaster) return false;

	float windowWidth=RayCaster->m_windowWidth;
	float windowCenter=RayCaster->m_windowCenter;

	mip_scale=1.0f/windowWidth;
	mip_shift=0.5-windowCenter/windowWidth;
	
	return true;
}



